@@ -7,7 +7,7 @@ import typing
 from sqlalchemy import orm
 
 __all__ = (
-    "DictRepository",
+    "DummyRepository",
     "Repository",
     "Resource",
     "SqlAlchemyRepository",
@@ -38,7 +38,10 @@ class Resource(abc.ABC, typing.Generic[T]):
     def __eq__(self, other: object) -> bool:
         if other.__class__ is self.__class__:
             # noinspection PyTypeChecker
-            return self.resource_name == typing.cast(Resource[typing.Any], other).resource_name
+            return (
+                self.resource_name
+                == typing.cast(Resource[typing.Any], other).resource_name
+            )
         else:
             return NotImplemented
 
@@ -63,6 +66,10 @@ class Repository(Resource[E], abc.ABC, typing.Generic[E]):
     def add(self, item: E, /) -> E:
         raise NotImplementedError
 
+    @abc.abstractmethod
+    def add_all(self, items: typing.Collection[E], /) -> typing.Collection[E]:
+        raise NotImplementedError
+
     @abc.abstractmethod
     def all(self) -> typing.Iterable[E]:
         raise NotImplementedError
@@ -71,6 +78,14 @@ class Repository(Resource[E], abc.ABC, typing.Generic[E]):
     def delete(self, item: E, /) -> E:
         raise NotImplementedError
 
+    @abc.abstractmethod
+    def delete_all(self) -> None:
+        raise NotImplementedError
+
+    @abc.abstractmethod
+    def set_all(self, items: typing.Collection[E], /) -> typing.Collection[E]:
+        raise NotImplementedError
+
     @abc.abstractmethod
     def update(self, item: E, /) -> E:
         raise NotImplementedError
@@ -81,7 +96,7 @@ class Repository(Resource[E], abc.ABC, typing.Generic[E]):
 
 
 class SqlAlchemyRepository(Repository[E], typing.Generic[E]):
-    def __init__(self, session: orm.Session):
+    def __init__(self, session: orm.Session, /):
         self._session = session
         self._entity_type: typing.Optional[typing.Type[E]] = None
 
@@ -89,6 +104,10 @@ class SqlAlchemyRepository(Repository[E], typing.Generic[E]):
         self.session.add(item)
         return item
 
+    def add_all(self, items: typing.Collection[E], /) -> typing.Collection[E]:
+        self.session.bulk_save_objects(items)
+        return items
+
     def all(self) -> typing.Generator[E, None, None]:
         return self.session.query(self.entity_type).all()
 
@@ -96,6 +115,9 @@ class SqlAlchemyRepository(Repository[E], typing.Generic[E]):
         self.session.delete(item)
         return item
 
+    def delete_all(self) -> None:
+        self.session.query(self.entity_type).delete(synchronize_session=False)
+
     def get(self, item_id: typing.Any, /) -> E:
         return self.session.query(self.entity_type).get(item_id)
 
@@ -108,12 +130,17 @@ class SqlAlchemyRepository(Repository[E], typing.Generic[E]):
     def rollback(self) -> None:
         self.session.rollback()
 
+    def save(self) -> None:
+        self.session.commit()
+
     @property
     def session(self) -> orm.Session:
         return self._session
 
-    def save(self) -> None:
-        self.session.commit()
+    def set_all(self, items: typing.Collection[E], /) -> typing.Collection[E]:
+        self.session.query(self.entity_type).delete()
+        self.session.bulk_save_objects(items)
+        return items
 
     def update(self, item: E, /) -> E:
         self.session.merge(item)
@@ -123,7 +150,7 @@ class SqlAlchemyRepository(Repository[E], typing.Generic[E]):
         return self.session.query(self.entity_type).filter(predicate).all()
 
 
-class DictRepository(Repository[E], abc.ABC, typing.Generic[E]):
+class DummyRepository(Repository[E], typing.Generic[E]):
     """Repository implementation based on a dictionary
 
     This exists primarily to make testing in client code simpler.  It was not designed for efficiency.
@@ -132,13 +159,13 @@ class DictRepository(Repository[E], abc.ABC, typing.Generic[E]):
     def __init__(
         self,
         *,
-        initial_values: typing.Dict[typing.Hashable, E],
+        initial_values: typing.Iterable[E],
         key_fn: typing.Callable[[E], typing.Hashable],
     ):
         super().__init__()
 
-        self._previous_state = initial_values
-        self._current_state = initial_values.copy()
+        self._current_state: typing.List[E] = list(initial_values)
+        self._previous_state: typing.List[E] = self._current_state.copy()
         self._key_fn = key_fn
 
         self.events: typing.List[typing.Tuple[str, typing.Dict[str, typing.Any]]] = []
@@ -153,29 +180,47 @@ class DictRepository(Repository[E], abc.ABC, typing.Generic[E]):
 
     def add(self, item: E, /) -> E:
         self.events.append(("add", {"item": item}))
-        key = self._key_fn(item)
-        self._current_state[key] = item
+        self._current_state.append(item)
         return item
 
+    def add_all(self, items: typing.Collection[E], /) -> typing.Collection[E]:
+        self.events.append(("add_all", {"items": items}))
+        self._current_state += items
+        return items
+
     def all(self) -> typing.Iterable[E]:
         self.events.append(("all", {}))
-        return list(self._current_state.values())
+        return list(self._current_state)
 
     def delete(self, item: E, /) -> E:
         self.events.append(("delete", {"item": item}))
-        key = self._key_fn(item)
-        del self._current_state[key]
+        self._current_state = [
+            o for o in self._current_state if self._key_fn(item) != self._key_fn(o)
+        ]
         return item
 
+    def delete_all(self) -> None:
+        self.events.append(("delete_all", {}))
+        self._current_state = []
+
+    def set_all(self, items: typing.Collection[E], /) -> typing.Collection[E]:
+        self.events.append(("set_all", {"items": items}))
+        self._current_state = items
+        return items
+
     def update(self, item: E, /) -> E:
         self.events.append(("update", {"item": item}))
-        key = self._key_fn(item)
-        self._current_state[key] = item
+        original_index = next(
+            ix
+            for ix, o in enumerate(self._current_state)
+            if self._key_fn(item) == self._key_fn(o)
+        )
+        self._current_state[original_index] = item
         return item
 
     def get(self, item_id: typing.Any, /) -> E:
         self.events.append(("get", {"item_id": item_id}))
-        return self._current_state[item_id]
+        return next(o for o in self._current_state if self._key_fn(o) == item_id)
 
 
 def _get_next_descendant_of(
